<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */

require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyCredentialsModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyLogger.php';

/**
 * ThreeByFourOneyMarketingMarketingWebService class
 * Oney's Marketing API interactions and manipulations
 *
 * @version 3.0.0
 *
 */
class ThreeByFourOneyMarketingWebService
{
    protected $business_transactions_url;
    protected $credentials;
    protected $logs_config;
    protected $simulations_url;
    protected $utility;

    /**
     * __construct function
     * Define Marketing API's URI and endpoints for staging and production environment
     *
     * @version 3.0.0
     * @return mixed
     */

    public function __construct()
    {
        $this->credentials = new ThreeByFourOneyCredentialsModel();
        $this->utility = new ThreeByFourOneyUtility();
    }

    /**
     * initPaymentWebService function
     * Init Web Services variables scoped by shop
     *
     * @version 1.1.0
     * @param int $shopId
     * @return mixed
     */
    private function initPaymentWebService($shopId)
    {
        $environnementConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT', null, null, $shopId);
        ($environnementConfig === 'staging') ? $dns = 'https://oney-staging.azure-api.net' : $dns = 'https://api.oney.io';
        $apiVersion = 'v1.1';
        // Business transactions
        $businessTransactionsEndPoint = '/product_catalog/' . $apiVersion . '/business_transaction';
        $businessTransactionsUrl = $dns . $businessTransactionsEndPoint;
        $this->business_transactions_url = $businessTransactionsUrl;
        // Simulations
        $simulationsEndPoint = '/sale_support_tools/' . $apiVersion . '/simulation';
        $simulationsUrl = $dns . $simulationsEndPoint;
        $this->simulations_url = $simulationsUrl;
        $this->logs_config = (string)Configuration::get('TBFONEY_ACTIVATE_LOGS', null, null, $shopId);
    }

    /**
     * verifyCredentials function
     * Verify merchant's credentials validity. Focus on HTTP response code. Cases are mapped in postProcessSaveCredentials [200, 401, 500]
     *
     * @version 3.0.0
     * @param string $environment
     * @param array $data
     * @param string $countryCode
     * @param int $shopId
     * @return array
     */
    public function verifyCredentials($environment, $data, $countryCode, $shopId)
    {
        $this->initPaymentWebService($shopId);
        if ($environment != null && $data != null && $countryCode != null)
        {
            $url = $this->business_transactions_url . '?' . 'merchant_guid=' . $data['merchant_guid'];
            if (!empty($environment) && !empty($data['merchant_guid']) && !empty($data['marketing_api_key']) && !empty($countryCode))
            {
                $response = $this->callOneyMarketingApi($url, $data['marketing_api_key'], $countryCode);
                if ($this->logs_config === 'yes')
                {
                    $log = new ThreeByFourOneyLogger();
                    $log->debug('[MARKETING API] VERIFY CREDENTIALS: API RESPONSE CODE: ' . $response['responseCode'] . ' - RESPONSE: ' . json_encode($response));
                    if (isset($response['body']->Errors) || isset($response['body']->StatusCode) || isset($response['body']->error_list))
                    {
                        $log->error('[MARKETING API] VERIFY CREDENTIALS: API RESPONSE CODE: ' . $response['body']->StatusCode . ' - API RESPONSE MESSAGE: ' . json_encode($response['body']->Errors[0]));
                    }
                }
                return $response;
            }
        }
    }
    
    /**
     * requestBusinessTransactionsCatalog function
     * Request Oney's Marketing API for retrieving Business Transactions catalog.
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param string $countryCode
     * @return mixed
     */
    public function requestBusinessTransactionsCatalog($environment, $countryCode)
    {
        $shopId = (int)Context::getContext()->shop->id;
        $this->initPaymentWebService($shopId);
        if ($environment != null && $countryCode != null)
        {
            $credentials = $this->credentials->getCredentials($environment, $countryCode, $shopId);
            if (!empty($credentials))
            {
                foreach ($credentials as $credential)
                {
                    $url = $this->business_transactions_url . '?' . 'merchant_guid=' . $credential['merchant_guid'];
                    if (!empty($credential['merchant_guid']) && !empty($credential['marketing_api_key']))
                    {
                        $response = $this->callOneyMarketingApi($url, $credential['marketing_api_key'], $countryCode);
                        $responseBody = $response['body'];
                        if ($this->logs_config === 'yes')
                        {
                            $log = new ThreeByFourOneyLogger();
                            $log->debug('[MARKETING API] REQUEST BUSINESS TRANSACTION CATALOG: API RESPONSE CODE: ' . $response['responseCode'] . ' - RESPONSE: ' . json_encode($response));
                            if (isset($responseBody->Errors) || isset($responseBody->StatusCode) || isset($responseBody->error_list))
                            {
                                $log->error('[MARKETING API] REQUEST BUSINESS TRANSACTION CATALOG: API RESPONSE CODE: ' . $responseBody->StatusCode . ' - API RESPONSE MESSAGE: ' . json_encode($responseBody->Errors[0]));
                            }
                        }
                        return json_decode($responseBody);
                    }
                }
            }
        }
    }

    /**
     * requestSimulations function
     * Request Oney's Marketing API for retrieving specific Business Transaction based on product price or cart amount.
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param float $paymentAmount
     * @param string $businessTransactionCode
     * @param string $countryCode
     * @return object
     */
    public function requestSimulations($environment, $paymentAmount, $businessTransactionCode, $countryCode)
    {
        $shopId = (int)Context::getContext()->shop->id;
        $this->initPaymentWebService($shopId);
        if ($environment != null && $paymentAmount != null && $businessTransactionCode != null && $countryCode != null)
        {
            $credentials = $this->credentials->getCredentials($environment, $countryCode, $shopId);
            if (!empty($credentials))
            {
                foreach ($credentials as $credential)
                {
                    $url = $this->simulations_url . '?' . 'merchant_guid=' . $credential['merchant_guid'] . '&' . 'payment_amount=' . $paymentAmount . '&' . 'business_transaction_code=' . $businessTransactionCode;
                    if (!empty($credential['merchant_guid']) && !empty($credential['marketing_api_key']))
                    {
                        $response = $this->callOneyMarketingApi($url, $credential['marketing_api_key'], $countryCode);
                        $responseBody = $response['body'];
                        if ($this->logs_config === 'yes')
                        {
                            $log = new ThreeByFourOneyLogger();
                            $log->debug('[MARKETING API] REQUEST BUSINESS TRANSACTION: API RESPONSE CODE: ' . $response['responseCode'] . ' - RESPONSE: ' . json_encode($response));
                            if (isset($responseBody->Errors) || isset($responseBody->StatusCode) || isset($responseBody->error_list))
                            {
                                $log->error('[MARKETING API] REQUEST BUSINESS TRANSACTION: API RESPONSE CODE: ' . $responseBody->StatusCode . ' - API RESPONSE MESSAGE: ' . json_encode($responseBody->Errors[0]));
                            }
                        }
                        return json_decode($responseBody);
                    }
                }
            }
        }
    }

    /**
     * parseHttpHeaders function
     * Parse API response headers and reformat them for further manipulation.
     *
     * @version 3.0.0
     * @param string $httpHeaders
     * @return array
     */
    private function parseHttpHeaders($httpHeaders)
    {
        if (!$httpHeaders) return array();
        
        $i = 0;
        $explodeHeaders = explode(PHP_EOL, $httpHeaders);
        $protocole = null;
        $reponseCode = null;
        $headers = [];

        while ($i !== count($explodeHeaders) - 1) {
            if ($explodeHeaders[$i] !== '')
            {
                if (strrpos($explodeHeaders[$i], 'HTTP') !== false) {
                    $explode = explode(' ', $explodeHeaders[$i]);
                    $protocole = $explode[0];
                    $reponseCode = $explode[1];
                    $responseCodeMessage = Tools::substr($explodeHeaders[$i], Tools::strlen($explode[0]) + Tools::strlen($explode[1]) + 2);
                } else {
                    $explode = explode(': ', $explodeHeaders[$i]);
                    if (isset($explode[1]) && !empty($explode[1])) {
                        $headers[$explode[0]] = $explode[1];
                    }
                }
            }
            $i++;
        }

        unset($explodeHeaders, $explode);

        return [
            'protocole' => $protocole,
            'responseCode' => $reponseCode,
            'responseCodeMessage' => $responseCodeMessage,
            'headers' => $headers,
        ];
    }

    /**
     * parseHttpResponse function
     * Parse API response body and reformat it for further manipulation.
     *
     * @version 3.0.0
     * @param string $response
     * @param array $headerSize
     * @return array
     */
    private function parseHttpResponse($response, $headerSize)
    {
        if (strpos($response, "HTTP") === false) throw new \Exception('This is not a valid HTTP callback.');

        $headers = Tools::substr($response, 0, $headerSize);

        $parsedHttp = $this->parseHttpHeaders($headers);
        $parsedHttp['body'] = Tools::substr($response, $headerSize);
        return $parsedHttp;
    }

    /**
     * callOneyMarketingApi function
     * CURL init and headers configuration for requesting Oney's APIs access.
     *
     * @version 3.0.0|3.1.16
     * @param string $url
     * @param string $marketingApiKey
     * @param string $countryCode
     * @return object|array
     */
    protected function callOneyMarketingApi($url, $marketingApiKey, $countryCode)
    {

        $curl = curl_init();
        // Handle country config with Oney's scoped countries
        $countryCode = $this->utility->getConfigByLanguage('iso_code', strtolower($countryCode));
        // HEADERS
        $headers = [
            'X-Oney-Authorization: ' . $marketingApiKey,
            'X-Oney-Partner-Country-Code: ' . $countryCode,
            'Content-Type: application/json; charset=utf-8'
        ];
        // OPTIONS
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_HEADER, true);
        curl_setopt($curl, CURLOPT_HTTPGET, true);
        curl_setopt($curl, CURLINFO_HEADER_OUT, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($curl, CURLOPT_TIMEOUT, 10);  // Set timeout in seconds
        // EXECUTE:
        $httpResponse = curl_exec($curl);
        $headerSize = curl_getinfo($curl, CURLINFO_HEADER_SIZE);
        $response = $this->parseHttpResponse($httpResponse, $headerSize);
        $responseBody = json_decode($response['body']);
        if ($this->logs_config === 'yes')
        {
            $log = new ThreeByFourOneyLogger();
            $log->debug('[MARKETING API] API RESPONSE CODE: ' . $response['responseCode'] . ' - API RESPONSE MESSAGE: ' . json_encode($response['responseCodeMessage']));
            if (isset($responseBody->Errors) || isset($responseBody->StatusCode) || isset($responseBody->error_list))
            {
                $log->error('[MARKETING API] API RESPONSE CODE: ' . $responseBody->StatusCode . ' - API RESPONSE MESSAGE: ' . json_encode($responseBody->Errors[0]));
            }
        }

        curl_close($curl);

        if ($response) return $response;
        return array();
    }
}
