<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */

require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyCredentialsModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyLogger.php';

/**
 * ThreeByFourOneyPaymentWebService class
 * Oney's Marketing API interactions and manipulations
 *
 * @version 3.0.0
 *
 */
class ThreeByFourOneyPaymentWebService
{
    protected $action_url;
    protected $credentials;
    protected $logs_config;
    protected $purchase_url;
    protected $utility;

    /**
     * __construct function
     * Define Payment API's URI and endpoints for staging and production environment
     *
     * @version 1.0.0
     * @return mixed
     */

    public function __construct()
    {
        $this->credentials = new ThreeByFourOneyCredentialsModel();
        $this->utility = new ThreeByFourOneyUtility();
    }

    /**
     * initMarketingWebService function
     * Init Web Services variables scoped by shop
     *
     * @version 1.1.0
     * @param int $shopId
     * @return mixed
     */
    private function initMarketingWebService($shopId)
    {
        $environmentConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT', null, null, $shopId);
        ($environmentConfig === 'staging') ? $dns = 'https://api-staging.oney.io' : $dns = 'https://api.oney.io';
        $apiVersion = 'v1.1';
        // Purchase
        $purchaseEndPoint = '/payments/' . $apiVersion . '/purchase/3x4x_context';
        $purchaseUrl = $dns . $purchaseEndPoint;
        $this->purchase_url = $purchaseUrl;
        // Actions (shared with confirm and cancel)
        $actionEndPoint = '/payments/' . $apiVersion . '/purchase';
        $actionUrl = $dns . $actionEndPoint;
        $this->action_url = $actionUrl;

        $this->logs_config = (string)Configuration::get('TBFONEY_ACTIVATE_LOGS', null, null, $shopId);
    }

    /**
     * verifyCredentials function
     * Request Oney's payment API for validating credentials.
     *
     * @version 3.0.2
     * @param string $environment
     * @param array $data
     * @param string $reference
     * @param string $countryCode
     * @param int $shopId
     * @return void
     */
    public function verifyCredentials($environment, $data, $reference, $countryCode, $shopId)
    {
        $this->initMarketingWebService($shopId);
        if ($environment != null && $data != null && $countryCode != null)
        {
            $url = $this->action_url . '/psp_guid/' . $data['psp_guid'] . '/merchant_guid/' . $data['merchant_guid'] . '/reference/' . $reference . '/status';
            $requestBody = [];
            $action = 'status';

            if (!empty($environment) && !empty($data['psp_guid']) && !empty($data['payment_api_key']) && !empty($countryCode))
            {
                $response = $this->callOneyPaymentApi($url, $data['payment_api_key'], $countryCode, $requestBody, $action);
                $responseBody = $response['body'];
                $decodedResponse = json_decode($responseBody);
                $field = [];

                if ($this->logs_config === 'yes')
                {
                    $log = new ThreeByFourOneyLogger();
                    $log->debug('[PAYMENT API] VERIFY CREDENTIALS: API RESPONSE CODE: ' . $response['responseCode'] . ' - RESPONSE: ' . json_encode($response));
                    if (isset($response['body']->Errors) || isset($response['body']->StatusCode) || isset($response['body']->error_list))
                    {
                        $log->error('[PAYMENT API] VERIFY CREDENTIALS: API RESPONSE CODE: ' . $response['body']->StatusCode . ' - API RESPONSE MESSAGE: ' . json_encode($response['body']->Errors[0]));
                    }
                }

                if (isset($decodedResponse->statusCode)) {
                    return $response['responseCode'];
                } elseif (isset($decodedResponse->error_list)) {
                    foreach ($decodedResponse->error_list as &$error)
                    {
                        $franceOverseasTerritoriesCodes = ['GP', 'GF', 'RE', 'MQ', 'YT', 'NC', 'PF', 'BL', 'MF', 'PM', 'WF'];
                        if ($countryCode == 'FR' || in_array($countryCode, $franceOverseasTerritoriesCodes))
                        {
                            $field = $error->error->error_code;
                            if ($field == 'ERR_09') return $response['responseCode'] = 400; else return $response['responseCode'] = 401;
                        } else {
                            $field = $error->error->field;
                            if ($field == 'psp_guid') return $response['responseCode'] = 401; else return $response['responseCode'] = 400;
                        }
                    }
                } else {
                    return $response['responseCode'] = 401;
                }
            }
        }
    }

    /**
     * requestPayment function
     * Handle payment request to Oney's payment API
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param string $countryCode
     * @param array $requestBody
     * @param int $shopId
     * @return array
     */
    public function requestPayment($environment, $countryCode, $requestBody, $shopId)
    {
        $this->initMarketingWebService($shopId);
        if ($environment != null && $countryCode != null)
        {
            $credentials = $this->credentials->getCredentials($environment, $countryCode, $shopId);
            if (!empty($credentials))
            {
                foreach ($credentials as $credential)
                {
                    $url = $this->purchase_url;
                    $action = 'payment';
                    $response = $this->callOneyPaymentApi($url, $credential['payment_api_key'], $countryCode, json_encode($requestBody), $action);
                    $responseBody = $response['body'];
                    if ($this->logs_config === 'yes')
                    {
                        $log = new ThreeByFourOneyLogger();
                        $log->debug('[PAYMENT API] REQUEST PAYMENT: URL: ' . $url);
                        $log->debug('[PAYMENT API] REQUEST PAYMENT: RESPONSE CODE: ' . $response['responseCode'] . ' - RESPONSE MESSAGE: ' . json_encode($response['responseCodeMessage']));
                        if (isset($responseBody->Errors) || isset($responseBody->StatusCode) || isset($responseBody->error_list))
                        {
                            $log->error('[PAYMENT API] REQUEST PAYMENT: RESPONSE CODE: ' . $responseBody->StatusCode . ' - RESPONSE MESSAGE: ' . json_encode($responseBody->Errors[0]));
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: ACTION: ' . $action);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: PSP GUID: ' . $credential['psp_guid']);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: MERCHANT GUID: ' . $credential['merchant_guid']);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: REQUEST BODY: ' . json_encode($requestBody));
                        }
                    }
                    return json_decode($responseBody);
                }
            }
        }
    }

    /**
     * requestAction function
     * Handle confirm and cancel actions. Request this method for order update
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param string $countryCode
     * @param string $action 'confirm' or 'cancel'
     * @param string $reference
     * @param array $requestBody
     * @return array|false
     */
    public function requestAction($environment, $countryCode, $action, $reference, $requestBody, $shopId)
    {
        $this->initMarketingWebService($shopId);
        if ($environment != null && $countryCode != null & $action != null && $reference != null && $requestBody != null)
        {
            $credentials = $this->credentials->getCredentials($environment, $countryCode, $shopId);
            if (!empty($credentials))
            {
                foreach ($credentials as &$credential)
                {
                    $url = $this->action_url . '/psp_guid/' . $credential['psp_guid'] . '/merchant_guid/' . $credential['merchant_guid'] . '/reference/' . $reference . '/action/' . $action;
                    $response = $this->callOneyPaymentApi($url, $credential['payment_api_key'], $countryCode, json_encode($requestBody), $action);
                    $responseBody = $response['body'];
                    if ($this->logs_config === 'yes')
                    {
                        $log = new ThreeByFourOneyLogger();
                        $log->debug('[PAYMENT API] REQUEST PAYMENT: RESPONSE CODE: ' . $response['responseCode'] . ' - RESPONSE MESSAGE: ' . json_encode($response['responseCodeMessage']));
                        if (isset($responseBody->Errors) || isset($responseBody->StatusCode) || isset($responseBody->error_list))
                        {
                            $log->error('[PAYMENT API] REQUEST PAYMENT: RESPONSE CODE: ' . $responseBody->StatusCode . ' - RESPONSE MESSAGE: ' . json_encode($responseBody->Errors[0]));
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: ACTION: ' . $action);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: PSP GUID: ' . $credential['psp_guid']);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: MERCHANT GUID: ' . $credential['merchant_guid']);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: ORDER REFERENCE: ' . $reference);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: REQUEST BODY: ' . json_encode($requestBody));
                        }
                    }
                    return json_decode($responseBody);
                }
            } else {
                if ($this->logs_config === 'yes')
                {
                    $log = new ThreeByFourOneyLogger();
                    $log->error('[PAYMENT API] REQUEST PAYMENT: MISSING CREDENTIAL(S) TO ENSURE THE REQUEST');
                }
            }
        } else {
            if ($this->logs_config === 'yes')
            {
                $log = new ThreeByFourOneyLogger();
                $log->error('[PAYMENT API] REQUEST PAYMENT: MISSING PARAMETER(S) TO ENSURE THE REQUEST');
                $log->error('[PAYMENT API] REQUEST PAYMENT: ENVIRONMENT: ' . $environment);
                $log->error('[PAYMENT API] REQUEST PAYMENT: COUNTRY: ' . $countryCode);
                $log->error('[PAYMENT API] REQUEST PAYMENT: ACTION: ' . $action);
                $log->error('[PAYMENT API] REQUEST PAYMENT: ORDER REFERENCE: ' . $reference);
                $log->error('[PAYMENT API] REQUEST PAYMENT: REQUEST BODY: ' . json_encode($requestBody));
            }
        }
        return false;
    }

    /**
     * requestStatus function
     * Handle the get status from Oney's payment API. Request this method to know at what status is exactly the interested order
     * You need to fill up the order reference to be understood by Oney's getStatus method.
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param string $countryCode
     * @param string $reference
     * @param int $shopId
     * @return object
     */
    public function requestStatus($environment, $countryCode, $reference, $shopId)
    {
        $this->initMarketingWebService($shopId);
        if ($environment != null && $countryCode != null && $reference != null)
        {
            $credentials = $this->credentials->getCredentials($environment, $countryCode, $shopId);
            if (!empty($credentials))
            {
                foreach ($credentials as $credential)
                {
                    $url = $this->action_url . '/psp_guid/' . $credential['psp_guid'] . '/merchant_guid/' . $credential['merchant_guid'] . '/reference/' . $reference . '/status';
                    $requestBody = [];
                    $action = 'status';
                    $response = $this->callOneyPaymentApi($url, $credential['payment_api_key'], $countryCode, $requestBody, $action);
                    $responseBody = $response['body'];
                    if ($this->logs_config === 'yes')
                    {
                        $log = new ThreeByFourOneyLogger();
                        $log->debug('[PAYMENT API] REQUEST PAYMENT: RESPONSE CODE: ' . $response['responseCode'] . ' - RESPONSE MESSAGE: ' . json_encode($response['responseCodeMessage']));
                        if (isset($responseBody->Errors) || isset($responseBody->StatusCode) || isset($responseBody->error_list))
                        {
                            $log->error('[PAYMENT API] REQUEST PAYMENT: RESPONSE CODE: ' . $responseBody->StatusCode . ' - RESPONSE MESSAGE: ' . json_encode($responseBody->Errors[0]));
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: ACTION: ' . $action);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: PSP GUID: ' . $credential['psp_guid']);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: MERCHANT GUID: ' . $credential['merchant_guid']);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: ORDER REFERENCE: ' . $reference);
                            $log->debug('[PAYMENT API] REQUEST PAYMENT: REQUEST BODY: ' . json_encode($requestBody));
                        }
                    }
                    return json_decode($responseBody);
                }
            }
        }
    }

    /**
     * parseHttpHeaders function
     * Parse API response headers and reformat them for further manipulation.
     *
     * @version 3.0.0
     * @param string $httpHeaders
     * @return array
     */
    private function parseHttpHeaders($httpHeaders)
    {
        if (!$httpHeaders) return array();
        
        $i = 0;
        $explodeHeaders = explode(PHP_EOL, $httpHeaders);
        $protocole = null;
        $reponseCode = null;
        $headers = [];

        while ($i !== count($explodeHeaders) - 1) {
            if ($explodeHeaders[$i] !== '')
            {
                if (strrpos($explodeHeaders[$i], 'HTTP') !== false) {
                    $explode = explode(' ', $explodeHeaders[$i]);
                    $protocole = $explode[0];
                    $reponseCode = $explode[1];
                    $responseCodeMessage = Tools::substr($explodeHeaders[$i], Tools::strlen($explode[0]) + Tools::strlen($explode[1]) + 2);
                } else {
                    $explode = explode(': ', $explodeHeaders[$i]);
                    if (isset($explode[1]) && !empty($explode[1])) {
                        $headers[$explode[0]] = $explode[1];
                    }
                }
            }
            $i++;
        }

        unset($explodeHeaders, $explode);

        return [
            'protocole' => $protocole,
            'responseCode' => $reponseCode,
            'responseCodeMessage' => $responseCodeMessage,
            'headers' => $headers,
        ];
    }

    /**
     * parseHttpResponse function
     * Parse API response body and reformat it for further manipulation.
     *
     * @version 3.0.0
     * @param string $response
     * @param array $headerSize
     * @return array
     */
    private function parseHttpResponse($response, $headerSize)
    {
        if (strpos($response, "HTTP") === false) throw new \Exception('This is not a valid HTTP callback.');

        $headers = Tools::substr($response, 0, $headerSize);

        $parsedHttp = $this->parseHttpHeaders($headers);
        $parsedHttp['body'] = Tools::substr($response, $headerSize);
        return $parsedHttp;
    }

    /**
     * callOneyPaymentApi function
     * CURL init and headers configuration for requesting Oney's APIs access.
     *
     * @version 3.0.0|3.1.16
     * @param string $url
     * @param string $paymentApiKey
     * @param string $countryCode
     * @param array $requestBody
     * @param string $action
     * @return array
     */
    protected function callOneyPaymentApi($url, $paymentApiKey, $countryCode, $requestBody, $action)
    {
        $curl = curl_init();
        // Handle country config with Oney's scoped countries
        $countryCode = $this->utility->getConfigByLanguage('iso_code', strtolower($countryCode));
        // Headers
        $headers = [
            'X-Oney-Authorization: ' . $paymentApiKey,
            'X-Oney-Partner-Country-Code: ' . $countryCode,
            'Content-Type: application/json; charset=utf-8',
            'X-Oney-Secret: None'
        ];
        // Options
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_HEADER, true);
        if ($action == 'status')
        {
            curl_setopt($curl, CURLOPT_HTTPGET, true);
            curl_setopt($curl, CURLINFO_HEADER_OUT, true);
        } else {
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $requestBody);
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($curl, CURLOPT_TIMEOUT, 10); // Set timeout in seconds
        // Execute
        $httpResponse = curl_exec($curl);
        $headerSize = curl_getinfo($curl, CURLINFO_HEADER_SIZE);
        $response = $this->parseHttpResponse($httpResponse, $headerSize);
        $responseBody = json_decode($response['body']);
        if ( $this->logs_config === 'yes')
        {
            $log = new ThreeByFourOneyLogger();
            $log->debug('[PAYMENT API] RESPONSE CODE: ' . $response['responseCode'] . ' - RESPONSE MESSAGE: ' . json_encode($response['responseCodeMessage']));
            if (isset($responseBody->error_list)) {
                foreach ($responseBody->error_list as $data) {
                    if (isset($data->error)) {
                        $log->error('[PAYMENT API] RESPONSE CODE: ' . $data->error->error_code . ' - RESPONSE MESSAGE: ' . $data->error->error_label);
                    }
                }
            }
            if (isset($responseBody->Errors) && isset($responseBody->StatusCode) && isset($responseBody->error_list))
            {
                $log->error('[PAYMENT API] RESPONSE CODE: ' . $responseBody->StatusCode . ' - RESPONSE MESSAGE: ' . json_encode($responseBody->Errors[0]));
            }
        }
        
        curl_close($curl);

        if ($response) return $response;
        return array();
    }
}
