<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyLogger.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/classes/ThreeByFourOneyPaymentWebService.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyOrdersModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyRefundsModel.php';

/**
 * AdminThreeByFourOneyCancellationController class
 * Admin controller focused on order cancellation.
 * 
 * @version 3.0.0
 */
class AdminThreeByFourOneyCancellationController
{
    protected $orders;
    protected $payment_web_service;
    protected $refunds;
    protected $utility;
    /**
     * __construct function
     * Define global variables and call dependencies.
     * 
     * @version 3.0.0
     * @return mixed
     */
    public function __construct()
    {
        $this->utility = new ThreeByFourOneyUtility();
        $this->payment_web_service = new ThreeByFourOneyPaymentWebService();
        $this->orders = new ThreeByFourOneyOrdersModel();
        $this->refunds = new ThreeByFourOneyRefundsModel();
    }

    /**
     * sendCancellation function
     * Manipulate amounts for making partial or total refunds
     * 
     * @version 3.1.17
     * @param int $idOrder
     * @param float $cancelAmount
     * @return array
     */
    public function sendCancellation($idOrder, $cancelAmount = null)
    {
        $order = new Order($idOrder);
        $orderCancellationAmount = number_format($order->total_paid, 2, '.', '');
        // $shippingCancellationAmount = number_format($order->total_shipping, 2, '.', '');
        $totalOrderAmount = (float)$orderCancellationAmount;
        $refunds = $this->refunds->getRefundsByOrderId($idOrder);
        // If cancelAmount params is null, it means the refund is total on the order
        if (is_null($cancelAmount)) {
            $cancelAmount = $totalOrderAmount;
            // Check if any previous refunds is recorded in oneygermany_refunds table
            if (!empty($refunds)) {
                foreach ($refunds as $refund) {
                    if ((float)$cancelAmount > (float)0) $cancelAmount -= (float)$refund['amount'];
                }
            }
            if ((float)$cancelAmount > (float)0) $this->requestCancellation($idOrder, $cancelAmount);
        } else {
            // If cancelAmount is NOT null, it means the refund is partial on the order
            $refundTotalAmount = (float)0;
            if (!empty($refunds)) {
                foreach ($refunds as $refund) $refundTotalAmount += (float)$refund['amount'];
            }
            // Special cases
            $currentAmountRemaining = $totalOrderAmount - $refundTotalAmount;
            if ($currentAmountRemaining > (float)0) {
                // Special case: cancelAmount params is equal or superior to currentAmountRemaining, then it means the refund is total on the order
                if ((float)$cancelAmount >= $currentAmountRemaining) $this->requestCancellation($idOrder, $currentAmountRemaining);
                // Special case: cancelAmount params is inferior to currentAmountRemaining, then it means the refund is partial on the order
                if ((float)$cancelAmount < $currentAmountRemaining) $this->requestCancellation($idOrder, $cancelAmount);
            }
        }
    }

    /**
     * requestCancellation function
     * Handle cancellation request between Prestashop and Oney's API.
     *
     * @version 1.1.0
     * @param int $idOrder
     * @param float $amount
     * @return array
     */
    private function requestCancellation($idOrder, $amount)
    {
        $order = new Order($idOrder);
        $shopId = (int)$order->id_shop;
        $environment = (string)Configuration::get('TBFONEY_ENVIRONMENT', null, null, $shopId);
        $reference = $order->reference;
        $billingAddress = new Address($order->id_address_invoice);
        $countryCode = (string)Country::getIsoById((int)$billingAddress->id_country);
        $logsConfig = (string)Configuration::get('TBFONEY_ACTIVATE_LOGS', null, null, $shopId);

        $requestBody = [
            'language_code' => strtoupper($countryCode),
            'merchant_request_id' => date('YmdHis') . '-' . $idOrder,
            'purchase' => [
                'cancellation_reason_code' => 0,
                'cancellation_amount' => number_format((float)$amount, 2, '.', '')
            ]
        ];

        $request = $this->payment_web_service->requestAction($environment, strtoupper($countryCode), 'cancel', $reference, $requestBody, $shopId);
        if (is_object($request))
        {
            $this->refunds->insertRefund(
                [
                    'order_id' => (int)$idOrder,
                    'amount' => number_format((float)$amount, 2, '.', '')
                ]
            );
        }
        if ($logsConfig === 'yes')
        {
            $log = new ThreeByFourOneyLogger();
            $log->debug('CANCELLATION REQUEST - ORDER REQUEST BODY: ' . json_encode($requestBody));
            $log->debug('CANCELLATION REQUEST - CANCELLATION AMOUNT: ' . number_format((float)$amount, 2, '.', ''));
        }

        return $request;
    }
}