<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyOrdersModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/classes/ThreeByFourOneyPaymentWebService.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyLogger.php';

/**
 * ThreeByFourOneyUpdateOrderStatusModuleFrontController class
 * Handle order statuses verification for Prestashop's order coherence with Oney's system.
 *
 * @version 3.0.0
 */
class ThreeByFourOneyUpdateOrderStatusModuleFrontController extends ModuleFrontController
{
    /**
     * postProcess function
     * Prestashop's post process function. Handle order statuses.
     *
     * @version 3.0.0
     * @return mixed
     */

    protected $shopId;
    protected $environment;
    protected $automatic;
    protected $orderModel;

    protected $favorableStatusId;
    protected $readyToDeliverStatusId;
    protected $outOfStockUnPaidStatusId;
    protected $paymentAcceptedStatusId;
    protected $partialRefundStatusId;
    protected $refundStatusId;
    protected $paymentErrorStatusId;
    protected $cancelledStatusId;
    protected $stocks;

    protected $reference;
    protected $countryCode;
    protected $orders;
    protected $payment_web_service;
    protected $utility;

    protected $logs;
    protected $allowLogs;

    private function recordLog($message)
    {
        if ($this->allowLogs) {
            $this->logs->debug($message);
        }
    }

    private function initialize()
    {
        $this->logs = new ThreeByFourOneyLogger();
        $this->payment_web_service = new ThreeByFourOneyPaymentWebService();
        $this->orderModel = new ThreeByFourOneyOrdersModel();
        $this->utility = new ThreeByFourOneyUtility();
    }

    private function loadConfiguration()
    {
        $this->shopId = (int)Context::getContext()->shop->id;
        $this->allowLogs = (string)Configuration::get('TBFONEY_ACTIVATE_LOGS', null, null, $this->shopId) === "yes";

        $this->environment = (string)Configuration::get('TBFONEY_ENVIRONMENT', null, null, $this->shopId);
        $this->automatic = (string)Configuration::get('TBFONEY_AUTOMATIC_CONFIRMATION', null, null, $this->shopId)  === 'yes';

        $this->favorableStatusId = (int)Configuration::get('TBFONEY_OS_FAVORABLE');
        $this->readyToDeliverStatusId = (int)Configuration::get('TBFONEY_OS_READYTODELIVER');
        $this->outOfStockUnPaidStatusId = (int)Configuration::get('PS_OS_OUTOFSTOCK_UNPAID');
        $this->paymentAcceptedStatusId = (int)Configuration::get('PS_OS_PAYMENT');
        $this->partialRefundStatusId = (int)Configuration::get('PS_CHECKOUT_STATE_PARTIAL_REFUND');
        $this->refundStatusId = (int)Configuration::get('PS_OS_REFUND');
        $this->paymentErrorStatusId = (int)Configuration::get('PS_OS_ERROR');
        $this->cancelledStatusId = (int)Configuration::get('PS_OS_CANCELED');
        $this->stocks = Configuration::get('PS_STOCK_MANAGEMENT', null, null, $this->shopId);

        if ($reference = Tools::getValue('reference')) {
            $this->orders = $this->orderModel->getOrderByReference($reference, Tools::getValue('iso_code'), $this->shopId);
        } else {
            $this->orders = $this->orderModel->getOrdersByShopId($this->shopId);
        }
    }

    private function saveHistory($orderId, $stateId, $finalize = false) {
        // 3.1.8 : Prevent an order to go back to a previous state
        if ($this->orderModel->checkStatus($orderId, $stateId)) {
            $history = new OrderHistory();
            $history->id_order = $orderId;
            $history->id_order_state = $stateId;
            $history->save();
        }
        
        // 3.1.8 : Finialize the order (no further update on status)
        if($finalize) {
            $this->orderModel->finalize($orderId);
        }
    }

    public function postProcess()
    {
        $this->initialize();
        $this->loadConfiguration();

        if ($this->orders)
        {
            foreach ($this->orders as $record)
            {
                // 3.1.8 : No treatment on final orders
                if ($record['final_state']) {
                    continue;
                }

                // 3.1.8 : Skip the order once to prevent too much calls to the API
                if ($record['initial_state']) {
                    $this->orderModel->initialize($record['id_order']);
                    continue;
                }

                $order = new Order($record['id_order']);

                $reference = $record['reference'];
                $countryCode = $record['country_code'];

                $hasStock = true;
                
                if ($this->stocks) {
                    foreach ($order->getCartProducts() as $product)
                    {
                        $stockAvailable = new StockAvailable();
                        $quantityAvailableByProduct = $stockAvailable->getQuantityAvailableByProduct($product['id_product'], $product['id_product_attribute'], $this->shopId);
                        $reservedStockQuantity = $this->orderModel->getStockReservedQuantity($product['id_product'], $product['id_product_attribute']);
                        
                        if (((int)$quantityAvailableByProduct < (int)$product['product_quantity']) && (int)$reservedStockQuantity <= (int)$product['product_quantity']) $hasStock = false;
                        if ((bool)$product['is_virtual']) $hasStock = true;
                    }
                } else {
                    $hasStock = false;
                }
                
                $requestStatusWebService = $this->payment_web_service->requestStatus($this->environment, $countryCode, $reference, $this->shopId);
                if (is_object($requestStatusWebService))
                {
                    if (isset($requestStatusWebService->purchase))
                    {
                        $statusCode = (string)$requestStatusWebService->purchase->status_code;
                        $statusMessage = $requestStatusWebService->purchase->status_label;
                        $this->recordLog('[CRON JOB] ORDER UPDATED: ' . $reference . ' - STATUS CALLBACK: ' . $statusCode . ' - MESSAGE: ' . json_encode($statusMessage));
                    } else {
                        if (isset($requestStatusWebService->error_list)) {
                            foreach ($requestStatusWebService->error_list as $error) {
                                $statusCode = (string)$error->error->error_code;
                                $statusMessage = $error->error->error_label;
                                $this->recordLog('[CRON JOB] ORDER UPDATED: ' . $reference . ' - STATUS CALLBACK: ' . $statusCode . ' - MESSAGE: ' . json_encode($statusMessage));
                            }
                        }
                    }

                    if (isset($statusCode) && $statusCode) {
                        switch ($statusCode) {

                            case 'REJECTED' :
                            case 'REFUSED' :
                                if ($order->current_state && ($order->current_state != $this->paymentErrorStatusId)) {
                                    $this->saveHistory($order->id, $this->paymentErrorStatusId, true);
                                }
                            break;

                            case 'ABANDONED' :
                            case 'ABORTED' :
                            case 'CANCELLED' :
                                if ($order->current_state && ($order->current_state != $this->cancelledStatusId)) {
                                    $this->saveHistory($order->id, $this->cancelledStatusId, true);
                                }
                            break;

                            case 'FAVORABLE' :
                                if ($order->current_state && ($order->current_state != $this->favorableStatusId)) {
                                    $this->saveHistory($order->id, $this->favorableStatusId);
                                }

                                // Special cases : if doesn't have stock and current status is favorable
                                // Force application of previous status (bypass saveHistory method)
                                if ($order->current_state && ($hasStock == false) && ($order->current_state == $this->favorableStatusId)) {
                                    $history = new OrderHistory();
                                    $history->id_order = $order->id;
                                    $history->id_order_state = $this->outOfStockUnPaidStatusId;
                                    $history->save();
                                    break;
                                }

                                // Special cases : if has stock and automatic confirmation config is yes
                                if (($hasStock == true) && $this->automatic) {
                                    $requestBody = [
                                        'language_code' => $countryCode,
                                        'merchant_request_id' => date('YmdHis') . '_' . $order->id_cart,
                                        'payment' => ['payment_amount' => number_format($record['amount'], 2, '.', '')]
                                    ];

                                    if ($this->payment_web_service->requestAction($this->environment, $countryCode, 'confirm', $reference, $requestBody, $this->shopId)) {
                                        if ($order->current_state && $order->current_state != $this->readyToDeliverStatusId) {
                                            $this->saveHistory($order->id, $this->readyToDeliverStatusId);
                                            // Cascade to payment accepted in hookActionOrderHistoryAddAfter
                                        }
                                    }
                                }
                            break;

                            case 'FUNDED' :
                                // Special case for France returning only funded
                                if (($hasStock == false) && $this->automatic && ($order->current_state != $this->favorableStatusId))
                                {
                                    $this->saveHistory($order->id, $this->favorableStatusId);
                                }

                                if ($order->current_state && ($order->current_state == $this->favorableStatusId))
                                {
                                    $history = new OrderHistory();
                                    $history->id_order = $order->id;
                                    $history->id_order_state = $this->outOfStockUnPaidStatusId;
                                    $history->save();
                                    break;
                                }

                                if (($hasStock == true) && $this->automatic)
                                {
                                    $requestBody = [
                                        'language_code' => $countryCode,
                                        'merchant_request_id' => date('YmdHis') . '_' . $order->id_cart,
                                        'payment' => ['payment_amount' => number_format($record['amount'], 2, '.', '')]
                                    ];
                                    if ($this->payment_web_service->requestAction($this->environment, $countryCode, 'confirm', $reference, $requestBody, $this->shopId)) {
                                        if ($order->current_state && ($order->current_state != $this->readyToDeliverStatusId)) {
                                            $this->saveHistory($order->id, $this->readyToDeliverStatusId);
                                            // Cascade to payment accepted in hookActionOrderHistoryAddAfter
                                        }
                                    }
                                    break;
                                }

                                if ($order->current_state && (($order->current_state != $this->paymentAcceptedStatusId) && ($order->current_state != $this->partialRefundStatusId) && ($order->current_state != $this->refundStatusId) && ($order->current_state != $this->outOfStockUnPaidStatusId))) {
                                    $this->saveHistory($order->id, $this->paymentAcceptedStatusId, true);
                                }
                            break;
                        }
                    }
                } else {
                    $this->recordLog('[CRON JOB] NO ORDER FOUND');
                }
            }
        } else {
            $this->recordLog('[CRON JOB] NO ORDER TO UPDATE');
        }
        die();
    }
}